/******************************************************************************
  Copyright (c) 1992 Xerox Corporation.  All rights reserved.
  Portions of this code were written by Stephen White, aka ghond.
  Use and copying of this software and preparation of derivative works based
  upon this software are permitted.  Any distribution of this software or
  derivative works must comply with all applicable United States export
  control laws.  This software is made available AS IS, and Xerox Corporation
  makes no warranty about the software, its performance or its conformity to
  any specification.  Any person obtaining a copy of this software is requested
  to send their name and post office or electronic mail address to:
    Pavel Curtis
    Xerox PARC
    3333 Coyote Hill Rd.
    Palo Alto, CA 94304
    Pavel@Xerox.Com
 *****************************************************************************/

#include "my-ctype.h"
#include "my-fcntl.h"
#include "my-stdio.h"
#include "my-unistd.h"

#include "config.h"
#include "log.h"
#include "network.h"
#include "server.h"
#include "streams.h"

static enum {STATE_OPEN, STATE_CLOSED} state = STATE_CLOSED;

const char *
network_protocol_name(void)
{
    return "single-user";
}

const char *
network_usage_string(void)
{
    return "";
}

int
network_initialize(int argc, char **argv)
{
    if (argc != 0)
	return 0;
    else
	return 1;
}

int
network_listen(void)
{
    return 1;
}

int
network_send_line(network_handle nh, const char *line, int flush_ok)
{
    printf("%s\n", line);
    fflush(stdout);

    return 1;
}

const char *
network_connection_name(network_handle nh)
{
    return "standard input";
}

int
network_set_connection_option(network_handle nh, const char *option, Var value)
{
    return 0;			/* no valid options (yet) */
}

void
network_close(network_handle nh)
{
    state = STATE_CLOSED;
}

void
network_shutdown(void)
{
    int flags;

    if ((flags = fcntl(0, F_GETFL)) < 0
	|| fcntl(0, F_SETFL, flags & ~NONBLOCK_FLAG) < 0)
	log_perror("Setting standard input blocking again");
}

static int	input_suspended = 0;

void
network_suspend_input(network_handle nh)
{
    input_suspended = 1;
}

void
network_resume_input(network_handle nh)
{
    input_suspended = 0;
}

int
network_process_io(unsigned timeout)
{
    network_handle 	nh;
    static server_handle sh;
    static Stream      *s = 0;
    char		buffer[1024];
    int			count;
    char	       *ptr, *end;
    int 		got_some = 0;
    
    if (s == 0) {
	int flags;
	
	s = new_stream(1000);

	if ((flags = fcntl(0, F_GETFL)) < 0
	    || fcntl(0, F_SETFL, flags | NONBLOCK_FLAG) < 0) {
	    log_perror("Setting standard input non-blocking");
	    return 0;
	}
    }

    switch (state) {
      case STATE_CLOSED:
	sh = server_new_connection(nh, 0);
	state = STATE_OPEN;
	got_some = 1;

      case STATE_OPEN:
	for (;;) {
	    while (!input_suspended
		   && (count = read(0, buffer, sizeof(buffer))) > 0) {
		got_some = 1;
		for (ptr = buffer, end = buffer + count; ptr < end; ptr++) {
		    unsigned char	c = *ptr;

		    if (isgraph(c)  ||  c == ' '  ||  c == '\t')
			stream_add_char(s, c);
		    else if (c == '\n')
			server_receive_line(sh, reset_stream(s));
		}
	    }

	    if (got_some  ||  timeout == 0)
		goto done;

	    sleep(1);
	    timeout--;
	}
    }

  done:
    return got_some;
}

char rcsid_net_single[] = "$Id: net_single.c,v 1.8 1992/10/23 23:03:47 pavel Exp $";

/* $Log: net_single.c,v $
 * Revision 1.8  1992/10/23  23:03:47  pavel
 * Added copyright notice.
 *
 * Revision 1.7  1992/10/21  03:02:35  pavel
 * Converted to use new automatic configuration system.
 *
 * Revision 1.6  1992/10/17  20:45:28  pavel
 * Changed to use generic support for POSIX-style non-blocking in "my-fcntl.h"
 * instead of a separate, different version.
 *
 * Revision 1.5  1992/10/06  18:20:11  pavel
 * Moved code to set stdin nonblocking out of the initialization routine and
 * into the first time that network_process_io() is called.
 * Also fixed up that routine to obviously return a value, circumventing a
 * warning from dumb compilers.
 *
 * Revision 1.4  1992/09/26  02:21:07  pavel
 * Added support for printing the network protocol name on server start-up.
 *
 * Revision 1.3  1992/09/11  21:23:52  pavel
 * Tracked change to network.h.
 *
 * Revision 1.2  1992/09/03  23:52:00  pavel
 * Added RCS Id and Log strings.
 */
