/******************************************************************************
  Copyright (c) 1992 Xerox Corporation.  All rights reserved.
  Portions of this code were written by Stephen White, aka ghond.
  Use and copying of this software and preparation of derivative works based
  upon this software are permitted.  Any distribution of this software or
  derivative works must comply with all applicable United States export
  control laws.  This software is made available AS IS, and Xerox Corporation
  makes no warranty about the software, its performance or its conformity to
  any specification.  Any person obtaining a copy of this software is requested
  to send their name and post office or electronic mail address to:
    Pavel Curtis
    Xerox PARC
    3333 Coyote Hill Rd.
    Palo Alto, CA 94304
    Pavel@Xerox.Com
 *****************************************************************************/

#ifndef Sym_Table_h
#define Sym_Table_h 1

#include "my-stdio.h"

#include "config.h"
#include "structures.h"

struct Names {
    unsigned	max_size;
    unsigned	size;
    const char **names;
};

struct Env {
    unsigned	size;
    Var		vars[1];
};

extern Names   *new_names(unsigned max_size);
extern Names   *new_builtin_names(void);
extern Names   *copy_names(Names *);
extern unsigned	find_or_add_name(Names **, const char *);
extern int	find_name(Names *, const char *);
extern void	free_names(Names *);

extern Env     *new_env(unsigned size);
extern Env     *copy_env(Env *);
extern void	free_env(Env *);
extern Var	env_ref(Env *, unsigned);
extern void	env_set(Env *, unsigned, Var);

extern void	write_env(FILE *, Names *, Env *);
extern int	read_env(FILE *, Names **, Env **);

/* Environment slots for built-in variables */
#define SLOT_NUM	0
#define SLOT_OBJ	1
#define SLOT_STR	2
#define SLOT_LIST	3
#define SLOT_ERR	4
#define SLOT_PLAYER	5
#define SLOT_THIS	6
#define SLOT_CALLER	7
#define SLOT_VERB	8
#define SLOT_ARGS	9
#define SLOT_ARGSTR	10
#define SLOT_DOBJ	11
#define SLOT_DOBJSTR	12
/* taken out to match code_elt.c #define SLOT_PREP	8 */
#define SLOT_PREPSTR	13
#define SLOT_IOBJ	14
#define SLOT_IOBJSTR	15

#define FIRST_USER_SLOT	16

#endif /* !Sym_Table_h */

/* $Log: sym_table.h,v $
 * Revision 1.4  1992/10/23  23:03:47  pavel
 * Added copyright notice.
 *
 * Revision 1.3  1992/10/21  03:02:35  pavel
 * Converted to use new automatic configuration system.
 *
 * Revision 1.2  1992/08/31  22:24:26  pjames
 * Changed some `char *'s to `const char *'
 *
 * Revision 1.1  1992/07/20  23:23:12  pavel
 * Initial RCS-controlled version.
 */
